#!/usr/bin/python3

import numpy as np
# da bismo crtali grafike u pajtonu, potrebna nam je 
# biblioteka matplotlib
from matplotlib import pyplot as plt

# matplotlib je izuetno slicna toolbox-u za crtanje u Matlabu
# gotovo sva podesavanja i opcije su ekvivalentni

# u ovom primeru hocemo da ncrtamo grafik funkcija sin(x) i cos(x)

# da bismo ncrtali grafik, prvo treba da preciziramo interval na kome
# crtamo i da definisemo podelu intervala

# definisemo velicinu figure koju prikazujemo u incima i broj tacaka po incu
# num - broj figure
# facecolor - boja pozadine
# edgecolor - boja rama oko crteza 
# frameon - prikzati ram, boolean
plt.figure(figsize=(8,6), dpi=80)

# podelu kreiramo ranije pomenutom funkcijom linspace iz paketa numpy
X = np.linspace(-np.pi, np.pi, 256, endpoint=True)
# zatim izracunavamo vrednosti funkcije u tackama podele
C, S = np.cos(X), np.sin(X)

# s obzirom da radimo sa numpy objektima, moramo da koristimo funkcije
# iz numpy biblioteke

# da bismo nacrtali grafik koristimo funkciju plot
# ciji je prvi argument nezavisno promenljiva, a drugi argument zavisno promenljiva

# crtamo funkciju kosinus plavom bojom sa linijom debljine 2.5 
plt.plot(X, C, color="blue", linewidth=2.5, linestyle="-", label="kosinus")
# crtamo funkciju sinus krsticima crvene boje
plt.plot(X, S, color="red", linewidth=2.5, linestyle="-", label="sinus")

# postavljamo granice za prikazivanje grafika po x osi
plt.xlim(-4.0, 4.0)

# postavljamo granice za prikazivanje grafika po x osi
plt.ylim(-1.0, 1.0)

# da bismo postavili natpise na osama, potrebno je da uhvatimo trenutno
# aktivnu osu
ax = plt.gca()
ax.xaxis.set_ticks_position('bottom')
ax.spines['bottom'].set_position(('data', 0))
ax.yaxis.set_ticks_position('left')
ax.spines['left'].set_position(('data', 0))


# obelezavamo tacaka na x osi
plt.xticks(np.linspace(-4,4,9, endpoint=True))

# obelezavamo tacaka na y osi
plt.yticks(np.linspace(-1,1,5,endpoint=True))

# drugi nacin za obelezavanje osa. umesto brojeva, mozemo da napisemo
# imena u latex formatu
# plt.xticks([-np.pi, -np.pi/2, 0, np.pi/2, np.pi], [r'$-\pi$', r'$-\pi/2$', r'$0$', r'$+\pi/2$', r'$+\pi$'])
# plt.yticks([-1, 0, +1], [r'$-1$', r'$0$', r'$+1$'])

# da bismo ubacili legendu, moramo uz svaki grafik da dodamo opciju label i da ga na taj
# nacin umenujemo

# prikazujemo legendu uz eksplicitno navodjenje lokacije
plt.legend(loc='upper left')

# matplotlib nam dozvoljava i da obelezavamo tacke od interesa 
# pretpostavimo da nam je vazna tacka 2pi/3 i da hocemo da obelezimo ponasanje 
# funkcija u toj tacki
t = 2*np.pi/3

# crtamo tacku (t, cos(t)) na grafiku 
plt.scatter([t, ], [np.cos(t), ], 50, color='blue')
# crtamo liniju koja spaja tacku i vrednost funkcije u tacki
plt.plot([t, t], [0, np.cos(t)], color='blue', linewidth=2.5, linestyle="--")
# potpisujemo ucrtanu tacku
plt.annotate(r'$cos(\frac{2\pi}{3})=-\frac{1}{2}$',
             xy=(t, np.cos(t)), xycoords='data',
             xytext=(-90, -50), textcoords='offset points', fontsize=16,
             arrowprops=dict(arrowstyle="->", connectionstyle="arc3,rad=.2"))

# crtamo tacku (t, sin(t)) na grafiku 
plt.scatter([t, ],[np.sin(t), ], 50, color='red')
# crtamo liniju koja spaja tacku i vrednost funkcije u tacki
plt.plot([t, t],[0, np.sin(t)], color='red', linewidth=2.5, linestyle="--")

# potpisujemo ucrtanu tacku
plt.annotate(r'$sin(\frac{2\pi}{3})=\frac{\sqrt{3}}{2}$',
             xy=(t, np.sin(t)), xycoords='data',
             xytext=(+10, +30), textcoords='offset points', fontsize=16,
             arrowprops=dict(arrowstyle="->", connectionstyle="arc3,rad=.2"))

# crtnaje funkcije i prikazivanje crteza korisniku nisu ista stvar
# ostaje samo jos da prikazemo crtez korisniku
plt.show()


# ako zelimo, mozemo i da sacuvamo nacrtani grafik
plt.savefig("grafik.png", dpi=72)
