#!/usr/bin/python3

import sys
import numpy as np
import scipy.interpolate as sc
import matplotlib.pyplot as plt

# scipy biblioteka podrzava i skup funkcija za interpolaciju

# pravimo podelu intervala
x = np.linspace(0, 4, 12)
# racunamo vrednosti funkcije u tackama podele
y = np.cos(x**2/3+4)
# stampamo dobijene parove
print(x,y)

# prikazujemo na grafiku dobijene tacke
plt.figure(num = 1)
plt.plot(x, y, 'o')
plt.show()

# na osnovu skupa uredjenih parova (x,y) zelimo da kreiramo
# interpolacioni polinom
# da bismo to postigli treba nam funkcija interp1d koja kao
# argumente ima vrednosti x i y i metod koji zeimo da primenimo
# prilikom izracunavanja interpolacionog polinoma

# racunamo linearni interpolacioni polinom
f1 = sc.interp1d(x, y, kind = 'linear')

# racunamo kubni interpolacioni polinom
f2 = sc.interp1d(x, y, kind = 'cubic')

# na grafiku prikazujemo dobijene interpolacione polinome

# prvo pravimo novu proizvoljnu podelu intervala
xnew = np.linspace(0, 4, 30)

# krairamo novu figuru
plt.figure(num = 2)
# na istom grafiku prokazujemo sva tri crteza
# to postizemo tako sto redom navodimo parove nezavisnih i zavisno promenljivih
# u pozivu funkcije plot
plt.plot(x, y, 'o', xnew, f1(xnew), '-', xnew, f2(xnew), '--')
# prikazujemo legendu
plt.legend(['data', 'linear', 'cubic'], loc='best')
# na kraju, prikazujemo crtez
plt.show()

# pored interpolacionih polinoma, scipy biblioteka ima podrsku i za splajnove

# pravimo novu podelu intervala
x = np.linspace(-3, 3, 50)
# unosimo beli sum u vrednosti funkcije
y = np.exp(-x**2) + 0.1*np.random.rand(50)

# prikazujemo dobijenu parove tacaka
plt.figure(num=3)
plt.plot(x,y,'ro', ms = 5)
plt.show()

# na osnovu kreiranog skupa tacaka iracunavamo splajn aprkocimaciju
spl = sc.UnivariateSpline(x,y)
# pravimo novu podelu intervala
xs = np.linspace(-3,3, 1000)
# crtamo novi grafiku
plt.figure(num=4)
plt.plot(xs, spl(xs), 'g', lw = 3)
plt.show()

# zapamtimo staru aproksimaciju
old = spl(xs)

# rucno menjanmo faktor usrednjavanja
spl.set_smoothing_factor(0.5)
plt.figure(num = 5)
plt.plot(xs, spl(xs), 'b', lw=3)
plt.show()

# na kraju priakzujemo sve na istom grafiku
plt.figure(num = 6)
plt.plot(x,y,'ro', xs, old, 'g', xs, spl(xs), 'b')
plt.show()


