#!/usr/bin/python3

import sys
from sympy import *
init_printing(use_unicode=True)

# sve sto smo do sada radili bilo je numericko izracunavanje
# ponekad to nije ono sto nam treba, jer numericko izracunavanje sa sobom
# donosi nezgodnu stvar, tj. implicitno se svi brojevi numericki aproksimiraju sa nekom
# tacnoscu
# na primer, sqrt(2) je iracionalan broj i kao takav ima beskonacno mnogo decimala
# s obzirom da je racunar ipak konacna masina, iracionalni brojevi su uvek aproksimirani 
# sirinom tipa podataka koji se koristi
# tj. i pre pocetka iracunavanja mi smo uneli  gresku u nas sistem

# simbolicko izracunavanje nam u tom slucaju moze pomoci. Svi brojevi i konstante se tumace kao
# apsolutno tacne vrednost i ne aproksimiraju se. simbolicko izracunavanje nije zasnovano na 
# numerickim pravilima i kao takvo ne poznaje koncept numericke greske

# definisemo racionalan broj, simbolicki
a = Rational(1,2)
# definisemo a kao racinalan broj klasicno
aa = 1.0/2 
# stampamo dobijeni rezultat
print(a)
# stampamo dobijeni rezultat
print(aa)

# na ovom primeru se ne vidi razlika izmedju simbolickog i numerickog izracunvanja, 
# ali primer sa nekim racionalni ili iracionalnim brojem to bolje ilustruje
b = Rational(1,3)
print(b)
# definisemo b kao realni broj
bb = 1.0/3.0
print(bb)
# ovde je b tacna vrednost, tj. 1/3
# a bb je aproksimirana vrednost sa konacno mnogo cifara

# pi je konstanta sa tacnom vrednoscu
print(pi)
# koju moemo aproksimirati sa prozivoljno mnogo cifara
print(pi.evalf(1000))

# mozemo i da racunamo apsolutno tacno
print(pi**2)
print(pi + exp(1))
# ali mozemo i da aproksimiramo izraze
print((pi+exp(1)).evalf())

# pored konacnih vrednosti sympy podrzava i koncept beskonacnosti. Klasa beskonacnosti se zove oo
print(oo > 9999)
# racunanje sa beskonacnostima je takodje podrzano
print(oo + 1)

# tacna vrednost korena
print(sqrt(2))
# aprkosimirana vrednost korena
x = sqrt(2)
print(x.evalf(100))

# racunamo zbir racionalnih brojeva
print(Rational(1,2) + Rational(1,3))
# racunamo bir racinalnog i iracionalnog broja
print(pi + Rational(1,2))

# poread ugradjenih simbola, mozemo da definisemo i svoje simbole i kasnije ih koristimo
# u izraima

# definisemo dva nova simbola
x = Symbol('x')
y = Symbol('y')
z = Symbol('z')

# simbolicki izracunvamo izraze
print(x + y + x - y)
# izracunavamo kvadrat binoma
print((x+6)**2)

# ovakav prikaz bas i nije reprezentativan. najcesce ovo prikzuje u razvijenom obliku
# da bismo razvili algebrasku jednacini koristi se funkcija expandtabs
print(expand((x+y)**2))
print(expand((x+y)**3))

# prilikom razvijanje, mozemo da naglasimo da treba da se koristi kompleksni domen
print(expand(x+y, complex=True))
# mozemo da razvijamo i trigonometrijske formule
print(expand(cos(x+y), trig=True))

# pored razvijanja, mozemo i da uproscavamo jednacine
print(simplify((x + x*y)/x))
print(simplify(sin(x)/cos(x)))

# pored uproscavanja biblioteka sympy podrzava i faktorisanje polinoma
# u tu svrhu koristi se funkcija factor
print(factor(x**3 - x**2 + x - 1))
print(factor(x**2*z + 4*x*y*z + 4*y**2*z))

# ukoliko je neophodno pretrazivanje faktora ili pronalazenje konkretnog faktora, bolje je
# koristiti funkciju factor_list koja vraca faktore spakovane u listu
print(factor_list(x**2*z + 4*x*y*z + 4*y**2*z))

# funkcija factor u opstem slucaju ne radi samo nad polinomima, vec se moze primenjivati i na druge
# klase funkcija
print(factor(cos(x)**2 + 2*cos(x)*sin(x) + sin(x)**2))

# pored funkcije factor, cesto se koristi i funkcija collect, koja grupise cinioce po stepenima

# definisemo izraz
expr = x*y + x - 3 + 2*x**2 - z*x**2 + x**3
# i na primer, zelimo da grupisemo izraz oko stepena promenljive x
# to postizemo funkcijom collect
collected_expr = collect(expr, x)
print(collected_expr)

# iz ovako grupisanog izraza, lako mozemo da saznamo koji se koeficijent nalazi uz koji stepen promenljive
# da bismo to postigli koristimo metod coeff

# na primer, da bismo dobili koeficijent uz x^2, treba da napisemo sledecu naredbu
print(collected_expr.coeff(x, 2))

# funkcija cancel se koristi za ponistavanje clanova u racionalnim funkcijama
print(cancel((x**2 + 2*x + 1)/(x**2 + x)))

# definisemo izraz
expr = 1/x + (3*x/2 - 2)/(x - 4)
# svodimo izraz na neskrativ obliku
print(cancel(expr))

# definisemo novi izraz
expr = (x*y**2 - 2*x*y*z + x*z**2 + y**2 - 2*y*z + z**2)/(x**2 - 1)
# svodimo izraz na neskrativ obliku
print(cancel(expr))
